# {{ package_name }}
## Installation
Run `pip install .` from the directory containing the file `setup.py`

## Usage
Run `python -m {{ package_name }} --help` to receive a full list of available options
