"""
This class does not do anything useful at runtime. Its only purpose is to provide type annotations.
Since sphinx does not support .pyi files (yet?), this is a .py file.
"""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:

    from sila2.client import ClientMetadata


class AuthorizationServiceClient:
    """
    This Feature provides access control for the implementing server.

    It specifies the SiLA Client Metadata for the access token, that has been provided by the
    AuthenticationService core Feature.
    """

    AccessToken: ClientMetadata[str]
    """
    Token to be sent with every call in order to get access to the SiLA Server functionality.
    """
